﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using iFileProxy.Services;
using Serilog;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.StaticFiles;

namespace iFileProxy.Controllers
{
    public class iProxyController : ControllerBase
    {
        private readonly TaskManager _taskManager;
        private readonly AppConfig _appConfig;
        private readonly DatabaseGateService _dbGate;

        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<iProxyController>();


        public iProxyController(TaskManager taskManager, AppConfig appConfig, DatabaseGateService databaseGateService)
        {
            _taskManager = taskManager;
            _appConfig = appConfig;
            _dbGate = databaseGateService;
        }

        [HttpPost]
        [Route("/AddOfflineTask")]
        public async Task<ActionResult<CommonRsp>> AddOfflineTask()
        {   
            return await _taskManager.AddTask(HttpContext) switch
            {
                TaskAddState.Success => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.Success, Message = "succ" }),
                TaskAddState.Fail => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.Fail, Message = "unkown error!" }),
                TaskAddState.ErrUrlRepeat => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrUrlRepeat, Message = "此url已经在任务队列中,请勿重复提交" }),
                TaskAddState.ErrTaskIdRepeat => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrTaskIdRepeat, Message = "TaskIdRepeat!!!" }),
                TaskAddState.ErrUrlInvalid => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrUrlInvalid, Message = "非法Url" }),
                TaskAddState.ErrDbFail => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrDbFail, Message = "数据库数据提交失败!" }),
                TaskAddState.ErrMaxParallelTasksLimit => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrMaxParallelTasksLimit, Message = "服务端并行数量达到上限!" }),
                TaskAddState.ErrFileSizeLimit => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrFileSizeLimit, Message = "文件大小无效!" }),
                TaskAddState.ErrFileNameForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrFileNameForbidden, Message = "文件名非法!" }),
                TaskAddState.ErrIPForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrIPForbidden, Message = "请求次数超过限制!" }),
                TaskAddState.ErrTargetHostForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrTargetHostForbidden, Message = "目标主机不在服务白名单内!" }),
                TaskAddState.ErrGetFileInfo => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrGetFileInfo, Message = "目标文件信息获取失败!" }),
                TaskAddState.ErrQueueLengthLimit => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrQueueLengthLimit, Message = "服务器任务队列已满 请稍候重试!" }),
                TaskAddState.Pending => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.Pending, Message = "已经添加到任务队列!" }),
                TaskAddState.ErrDisabledStreamTransferOrZeroSize => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrDisabledStreamTransferOrZeroSize, Message = "禁止0大小文件或者流式传输!" }),
                TaskAddState.ErrKeywordForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.ErrKeywordForbidden, Message = "禁止代理此url!" }),
                _ => (ActionResult<CommonRsp>)Ok(new CommonRsp() { Retcode = (int)TaskAddState.Success, Message = "succ default" }),
            };
        }

        [HttpPost]
        [HttpGet]
        [Route("/GetMyTasks")]
        public ActionResult<CommonRsp> GetMyTasks()
        {
            var data = _taskManager.GetTaskListByIpAddr(HttpContext);
            foreach (var d in data)
            {
                if (d.Status == TaskState.Queuing)
                {
                    d.QueuePosition = _taskManager.GetQueuePosition(d.TaskId);
                }
            }
            return Ok(new CommonRsp() { Retcode = 0,  Data = data ,Message = "succ" });
        }

        [HttpPost]
        [HttpGet]
        [Route("/GetServerLoad")]
        public ActionResult<CommonRsp> GetServerLoad() {
            return Ok(new CommonRsp { Retcode = 0, Message = "succ", Data = _taskManager.GetServerTaskLoadInfo() });
        }

        [HttpGet]
        [HttpPost]
        [Route("/Download/{taskID}")]
        public async Task<IActionResult> DownloadFile(string taskID)
        {
            try
            {
                string? fileName = null;
                var d = _taskManager.GetTaskListByIpAddr(HttpContext);
                var taskInfo = _taskManager.GetTaskInfo(taskID);

                if ((!_appConfig.SecurityOptions.AllowDifferentIPsForDownload && d.Where(x => x.TaskId == taskID).Any()) || taskInfo?.Count > 0)
                {
                    if (_appConfig.SecurityOptions.AllowDifferentIPsForDownload)
                    {
                        if (taskInfo == null || taskInfo.Count == 0)
                        {
                            return Ok(new CommonRsp() { Message = "task not exists", Retcode = -1 });
                        }
                        fileName = taskInfo[0].FileName;
                    }
                    else
                    {
                        fileName = d.Where(x => x.TaskId == taskID).FirstOrDefault()?.FileName;
                    }

                    if (fileName == null)
                    {
                        return Ok(new CommonRsp() { Message = "file not exists or taskId error", Retcode = -1 });
                    }

                    var filePath = Path.Combine(_appConfig.DownloadOptions.SavePath, fileName);
                    if (!MasterHelper.CheckDownloadFileExists(fileName))
                    {
                        return Ok(new CommonRsp() { Message = "file not exists", Retcode = 1 });
                    }

                    // 获取文件的 MIME 类型
                    var provider = new FileExtensionContentTypeProvider();
                    string contentType;
                    if (!provider.TryGetContentType(filePath, out contentType))
                    {
                        contentType = "application/octet-stream";
                    }

                    // 获取文件信息
                    var fileInfo = new FileInfo(filePath);
                    
                    // 设置响应头
                    Response.Headers.Append("Content-Disposition", $"attachment; filename=\"{fileName}\"");
                    Response.Headers.Append("Content-Length", fileInfo.Length.ToString());

                    // 使用 FileStream 进行流式传输
                    var fileStream = new FileStream(
                        filePath, 
                        FileMode.Open, 
                        FileAccess.Read, 
                        FileShare.Read,
                        bufferSize: 81920,  // 80KB 缓冲区
                        useAsync: true
                    );

                    // 记录下载历史
                    await _dbGate.AddDownloadHistoryAsync(taskID, MasterHelper.GetClientIPAddr(HttpContext));

                    // 返回文件流
                    return new FileStreamResult(fileStream, contentType)
                    {
                        EnableRangeProcessing = true,  // 支持断点续传
                        FileDownloadName = fileName
                    };
                }

                return Ok(new CommonRsp() { Message = "task_id not exists", Retcode = -1 });
            }
            catch (Exception ex)
            {
                _logger.Error("处理下载请求失败: {ex}", ex);
                return Ok(new CommonRsp() 
                { 
                    Message = $"download failed: {ex.Message}", 
                    Retcode = -1 
                });
            }
        }
    }
}
