﻿using Newtonsoft.Json;
using Serilog;

namespace iFileProxy.Helpers
{

    public class CommandLineArgsHelper(string[] args)
    {
        private readonly Dictionary<string, string> _parameters = ParseArgs(args);
        
        // 解析命令行参数
        private static Dictionary<string, string> ParseArgs(string[] args)
        {
            var parameters = new Dictionary<string, string>();

            for (int i = 0; i < args.Length; i++)
            {
                var arg = args[i];

                // 如果参数是键值对形式
                if (arg.Contains('='))
                {
                    var parts = arg.Split(['='], 2);
                    if (parts.Length == 2)
                    {
                        parameters[parts[0].TrimStart('-')] = parts[1];
                    }
                }
                else
                {
                    // 如果参数是没有值的标志性参数（比如 --dev-logger）
                    parameters[arg.TrimStart('-')] = "true";
                }
            }

            return parameters;
        }

        // 获取指定参数的值，若无值则返回null
        public string? GetStringValue(string key)
        {
            return _parameters.ContainsKey(key) ? _parameters[key] : null;
        }

        // 获取指定参数的布尔值，若参数值是 "true" 则返回true，否则返回false
        public bool GetBooleanValue(string key)
        {
            return _parameters.ContainsKey(key) && _parameters[key].Equals("true", StringComparison.CurrentCultureIgnoreCase);
        }
    }
}
