﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using System.Text.Json;

namespace iFileProxy.Middlewares
{
    public class IPAccessLimitMiddleware(RequestDelegate next, Dictionary<string, Dictionary<string, uint>> IPAccessCountDict, AppConfig appConfig)
    {
        private readonly RequestDelegate _next = next;
        private readonly Dictionary<string, Dictionary<string, uint>> _IPAccessCountDict = IPAccessCountDict;

        public async Task InvokeAsync(HttpContext context)
        {
            string ipStr = MasterHelper.GetClientIPAddr(context);

            // 处理配置文件拉黑IP
            if (appConfig.SecurityOptions.BlockedClientIP.IndexOf(ipStr) != -1)
            {
                context.Response.StatusCode = 403;
                await context.Response.WriteAsJsonAsync(new CommonRsp { Retcode = 403, Message = "你的IP地址已经被管理员拉入黑名单!" });
                return;
            }

            // 获取需要跟踪的路由列表
            var routesToTrack = appConfig.SecurityOptions.RoutesToTrack;

            // 如果没有匹配到需要跟踪的路由，直接调用下一个中间件
            if (!routesToTrack.Any(p => context.Request.Path.ToString().StartsWith(p, StringComparison.OrdinalIgnoreCase)))
            {
                await _next(context);
                return;
            }

            string dateStr = DateTime.Now.ToString("yyyy-MM-dd");

            if (!_IPAccessCountDict.TryGetValue(dateStr, out var ipCounts))
            {
                ipCounts = [];
                _IPAccessCountDict[dateStr] = ipCounts;
            }

            if (ipCounts.TryGetValue(ipStr, out uint value))
            {
                if (ipCounts[ipStr] >= appConfig.SecurityOptions.DailyRequestLimitPerIP)
                {
                    context.Response.StatusCode = 200;
                    context.Response.ContentType = "application/json";
                    await context.Response.WriteAsync(JsonSerializer.Serialize(new CommonRsp { Retcode = 403, Message = "日请求次数超过限制! (GMT+8 时间00:00重置)" }));
                    return;
                }
                ipCounts[ipStr] = ++value;
            }
            else
            {
                ipCounts.Add(ipStr, 1);
            }

            await _next(context);
        }
    }
}
